/*
 * $Id: ActorInitializer.d,v 1.1.1.1 2003/11/28 17:26:30 kenta Exp $
 *
 * Copyright 2003 Kenta Cho. All rights reserved.
 */
module abagames.util.ActorInitializer;

/**
 * Pass initial parameters to the actor.
 */
//public interface ActorInitializer {
public abstract class ActorInitializer {
}
